export default {
    props: ['layout','group'],
    data: function () {
        return {
        }
    },
    components: {
        //'ptc-fw-panel': ptcFwPanel,
    },
    mounted() {
        this.$emit('mounted',{
            instance: this
        });
    },
    computed: {
    },
    methods: {
        /**
         * De-bounced version of forceUpdate.
         */
        delayedUpdate: IMatch.debounce(self => {
            self.$forceUpdate();
        },200),

        /**
         * Return a unique id for the given file by combining the
         * group id and the file id.
         * NOTE: A file may appear multiple times in a fw-grid so
         * we have to use the group id.
         * @param {Object} file The file to produce the id for
         */
        getFileId(file) {
            return `g${this.group.id}-c${file.id}`;
        },

        /**
         * Create an in-memory SVG with a size that exactly matches
         * the image dimensions. A perfect placeholder for lazy-loading.
         * @param {Object} file 
         */        
        getPlaceholderUrl(file)
        {
            return `data:image/svg+xml,%3Csvg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 ${file.imageWidth} ${file.imageHeight}"%3E%3C/svg%3E`;
        },

        /**
         * Return the url for the image for the given file.
         * The image size is controlled by the layout.
         * If the file is proteded and the display of protected files
         * is off, return a stock image symbolizing a protected file.
         * @param {*} file 
         */
        getImageUrl(file)
        {
            if (!file.protected || this.layout.showProtectedFiles) {
                return IMatch.getImageUrl(file.id,this.layout.imageSize);
            }
            else {
                return 'img/fw-shield-icon.svg';
            }
        },

        /**
         * Creates an observer for all elements matching the observedClass
         * under parent. When an observed element becomes visible, the
         * {@link onObserverCompleted} method is called.
         * Call this method whenever you change the DOM to restart watching
         * @param {*} parentElement 
         * @param {*} observerdClass 
         */
        createObserver(parentElement, observerdClass = '.ptc-fw-observed-image')
        {
            let self = this;

            if (this.observer) {
                this.observer.disconnect();
                this.observer = null;
            }

            // See: https://developers.google.com/web/fundamentals/performance/lazy-loading-guidance/images-and-video/
            this.observer = new IntersectionObserver(entries => {
                for (let e of entries) {
                    if (e.isIntersecting) {
                        let img = $(e.target);   
                        self.observer.unobserve(img[0]);
                        self.onObserverCompleted(img);
                    }
                }
            });

            // We grab all images in the group produced by this component 
            // and add them to the observer.  
            $(`${parentElement} ${observerdClass}`).each(function(e) {
                self.observer.observe(this);
            });
        },

        /**
         * This is called by the observer when the image has become visible.
         * If the layout uses square thumbns, it apples the data-src of img
         * to the background-image url, else it sets src to data-src.
         * @param {jQuery img} img The image that has become visible.
         */
        onObserverCompleted(img)
        {
            if (this.layout.squareThumbs) {
                img.css('background-image', `url(${img.attr('data-src')})`);
            }
            else {
                img.attr('src',img.attr('data-src'));
            }
            img.addClass('animated fadeIn');
        },

        /**
         * Return true if the specified element is in view (visble)
         * @param {*} elem 
         */
        inView(elem)
        {
            let e = $(elem);
            if (!e.length) {
                return false;  // Not in DOM (yet)
            }

            let wh = $(window).height();
            let viewTop = $(window).scrollTop();
            let viewBottom = viewTop + wh;
        
            var elemTop = e.offset().top;
            var elemBottom = elemTop + e.height();
        
            // Conider full visible (elem inside view) and
            // parsial visibility (elem partially visible at the top or bottom)

            return ((elemTop >= viewTop) && (elemBottom <= viewBottom)) ||  // Completely in View
                   ((elemBottom >= viewTop) && (elemTop <= viewTop)) ||  // Partial Above
                   ((elemBottom >= viewBottom) && (elemTop <= viewBottom)) || // Partial Below
                   ((elemTop <= viewBottom+wh) && (elemTop >= viewTop)); // Below the view, within one window height (pre-load of invisible images)
                   
        },

        getHeader(file)
        {
            if (this.layout.header  && this.layout.header.visible) {
                if (this.layout.header.contentFunction != undefined) {
                    return this.layout.header.contentFunction(file,this.layout);
                }
            }
            return null;
        },

        getFooter(file)
        {
            if (this.layout.footer && this.layout.footer.visible) {
                if (this.layout.footer.contentFunction != undefined) {
                    return this.layout.footer.contentFunction(file,this.layout);
                }
            }
            return null;
        },

        getListContent(file)
        {
            if (this.layout.listData && this.layout.listData.visible) {
                if (this.layout.listData.contentFunction != undefined) {
                    return this.layout.listData.contentFunction(file,this.layout);
                }
            }
            return null;
        },

        clearSelection(clearFocus)
        {
            if (clearFocus || false) {
                this.clearFocus();
            }
            
            if (this.layout.selection.length > 0) {
                this.layout.selection = [];
                this.$emit('selection-changed');
            }
        },

        clearFocus()
        {
            if (this.layout.focusedFile != 0) {
                this.layout.focusedFile = 0;
                this.$emit('focus-changed');
            }
        },

        setFocus(id)
        {
            if (this.layout.focusedFile != id) {
                this.layout.focusedFile = id;
                this.$emit('focus-changed');
            }
        },

        select(id,set)
        {   
            let i = this.layout.selection.indexOf(id);
            if (i >= 0) {
                if (!set) {
                    this.layout.selection.splice(i,1);
                    this.$emit('selection-changed');
                }
            }
            else {
                if (set) {
                    this.layout.selection.push(id);
                    this.$emit('selection-changed');
                }
            }
        },

        isSelected(id)
        {
            return this.layout.selection.indexOf(id) >= 0;
        },

        isFocused(id)
        {
            return this.layout.focusedFile === id;
        },

        getSelectionClass(id)
        {
            if (this.layout.focusedFile === id) {
                return 'focused';
            }
            else if (this.layout.selection.indexOf(id) >= 0) {
                return 'selected';
            }
            else {
                return '';
            }
        },

        drillDown(group)
        {
            this.$emit('drill-down',group);
        },

        drillUp()
        {
            this.$emit('drill-up');
        },


        /**
         * Return the 0-based index of the focused file.
         */
        _getFocusIndex()
        {
            for (let i = 0; i < this.group.files.length; ++i) {
                if (this.group.files[i].id == this.layout.focusedFile) {
                    return i;
                }
            }

            return -1;
        },
        
        /**
         * Called by derived components when a file item was clicked.
         * @param {*} id 
         * @param {*} event 
         */
        itemClicked(id,event)
        {
            if (this.layout.allowSelection) {
                if (!this.layout.multiSelection || (!event.shiftKey && !event.ctrlKey)) {
                    this.clearSelection();
                }

                // Ctrl toggles selection of individual items.
                // Shift selects from the last clicked to the current clicked.
                if (event.shiftKey) {
                    if (this.layout.focusedFile != 0) {
                        
                        let last = -1;
                        let clicked = -1;
                        let cur;
                        for (let i = 0; i < this.group.files.length; ++i) {
                            cur = this.group.files[i].id;
                            if (cur == this.layout.focusedFile) {
                                last = i;
                            }
                            if (cur == id) {
                                clicked = i;
                            }
                            if ((last >= 0) && (clicked >= 0)) break;
                        }

                        if ((last >= 0) && (clicked >= 0)) {
                            this.clearSelection();                            
                            if (clicked < last) {
                                for (let i = clicked; i <= last; ++i) {
                                    this.select(this.group.files[i].id,true);
                                }
                            }
                            else {
                                for (let i = last; i <= clicked; ++i) {
                                    this.select(this.group.files[i].id,true);
                                }
                            }
                        }
                    }
                    else {
                        this.select(id,!this.isSelected(id));
                    }
                }
                else {
                    this.select(id,!this.isSelected(id));
                }
            }
            
            if (!event.shiftKey) {
                this.setFocus(id);
            }
        },

        /**
         * Navigate the focused item into the specified direction.
         * @param {*} where 
         */
        navigate(where)
        {
            let fi = this._getFocusIndex();
            let newId = null;

            switch (where) {
            case 'f':
                // first
                if (fi > 0) {
                    newId = this.group.files[0].id;
                }
                break;

            case 'l':
                // last
                if (fi >= 0) {
                    newId = this.group.files[this.group.files.length-1].id;
                }
                break;

            case 'p':
                // Previous            
                if (fi > 0) {
                    newId = this.group.files[fi-1].id;
                }
                break;

            case 'n':
                // Next
                if ((fi >= 0) && (fi+1 < this.group.files.length)) {
                    newId = this.group.files[fi+1].id;
                }
                break;
            }

            if (newId != null) {
                this.clearSelection();
                this.setFocus(newId);
                this.select(newId,true);
                this.onFocusChanged();
            }
        },

        onFocusChanged()
        {

        }

    },

    /*
        The outer <div> uses a unique id based on the group.id and the 'name' of the component.
        Example: <div :id="'fw-smart-'+group.id" class="fw-smart-container">
        This allows us to access it easily with jQuery.

        Placeholder and lazy-loading:

        Initially, each image uses a placeholder image based on an inline SVG image that is 
        sized to have the same dimensions as the thumbnail. We know these dimensions from the 
        imageHeight/imageWidth we request from the database.
        This src url is produced per-file with the getPlaceholderUrl() method.
        The data-src is set to the real image URL of the thumbnail.

        The component calls createObserver after the DOM is complete to watch for images
        coming into view. The observer triggers the onObserverCompleted method.

        Make sure to add the class '.ptc-fw-observed-image' to the background image and <img>.

    */
    template: ''
}


